package mark0;

import simudyne.core.abm.AgentBasedModel;
import simudyne.core.abm.Group;

public class Mark0Model extends AgentBasedModel<Globals> {

    @Override
    public void init() {
        createDoubleAccumulator("Unemployment");
        registerAgentTypes(Economy.class, Firm.class);
        registerLinkTypes(Links.EconomyToFirmLink.class, Links.FirmToEconomyLink.class);
        registerMessageTypes(Messages.OutputMessage.class, Messages.PriceMessage.class, Messages.WageMessage.class,
                Messages.AveragePriceMessage.class, Messages.AverageWageMessage.class,
                Messages.EmploymentUnemploymentMessage.class, Messages.DemandMessage.class,
                Messages.AvailableWorkersMessage.class, Messages.ProfitMessage.class, Messages.DividendMessage.class,
                Messages.DebtMessage.class, Messages.CashMessage.class, Messages.TotalDebtMessage.class,
                Messages.TotalCashMessage.class, Messages.ProductivityMessage.class,
                Messages.BailoutRequestMessage.class, Messages.DepositsMessage.class,
                Messages.HealthyFirmAccountMessage.class, Messages.BailoutPackageMessage.class,
                Messages.BankruptcyMessage.class);
    }

    public void setup() {
        Group<Firm> firmGroup = generateGroup(Firm.class, (int) getGlobals().N_f);
        Group<Economy> economyGroup = generateGroup(Economy.class, 1);

        firmGroup.fullyConnected(economyGroup, Links.FirmToEconomyLink.class);
        economyGroup.fullyConnected(firmGroup, Links.EconomyToFirmLink.class);

        super.setup();
    }

    @Override
    public void step() {
        super.step();

        // Initialization
        if (getContext().getTick() == 0) {
            run(Firm.initVariables());
            run(Firm.sendDeposits(), Economy.initSavings());
            run(Firm.sendWage(), Economy.initWageMap());
            run(Firm.sendPrice(), Economy.initPriceMap());
            run(Firm.sendProductivity(), Economy.initProductivityMap());
        }

        // Economy KPIs
        run(Firm.sendOutput(), Economy.calcEmploymentUnemployment());
        run(Economy.registerUnemployment());
        run(Economy.sendEmploymentUnemployment(), Firm.receiveEmploymentUnemployment());
        run(Firm.sendPrice(), Economy.calcAveragePrice());
        run(Economy.sendAveragePrice(), Firm.receiveAveragePrice());
        run(Firm.sendWage(), Economy.calcAverageWage());
        run(Firm.sendWage(), Economy.receiveWageMessages());
        run(Firm.sendProductivity(), Economy.receiveProductivity());
        run(Economy.calcAvailableWorkers(), Firm.receiveAvailableWorkers());

        if (getContext().getTick() > 0){
            run(Economy.calcInflation());
        }

        // Firm strategies
        run(Firm.calculateStrategy());
        run(Firm.sendOutput(), Economy.calcEmploymentUnemployment());
        run(Economy.sendEmploymentUnemployment(), Firm.receiveEmploymentUnemployment());
        run(Firm.sendPrice(), Economy.calcAveragePrice());
        run(Economy.sendAveragePrice(), Firm.receiveAveragePrice());


        // Household Demand
        run(Firm.sendWage(), Economy.calcTotalConsumptionBudget());
        run(Firm.sendPrice(), Economy.receivePriceMessage());
        run(Economy.calcDemand(), Firm.receiveDemand());

        // Firm Accounting
        run(Firm.doAccounting());
        run(Firm.sendProfit(), Economy.receiveProfit());
        run(Firm.sendHealthyFirmAccount(), Economy.receiveHealthyFirmAccounts());

        // Firm Defaults
        run(Firm.sendBailoutRequest(), Economy.receiveIndebtedFirmDebt());
        run(Economy.checkDefaults());
        run(Economy.sendHealthyFirmDeposits(), Firm.receiveHealthyFirmDeposits());
        run(Economy.sendBailoutPackages(), Firm.receiveBailoutPackage());
        run(Economy.sendBankruptcyMessages(), Firm.receiveBankruptcyMessage());

        // Firm Revivals
        run(Firm.doRevival());
        run(Firm.sendDebt(), Economy.receiveDebt());
        run(Firm.sendCash(), Economy.receiveCash());

        // Distribute Debt
        run(Economy.distributeDebt(), Firm.receiveCash());
    }
}
